;*************************************
;*          PROGRAM    MCS51         *
;*          XSA51 COMPLIER           *
;*************************************
                 ;
STATUS0          EQU    20H         ;1
STATUS           EQU    21H         ;1
STAS_KEY1        EQU    22H         ;1 byte  flag status1
STAS_KEY2        EQU    23H         ;1 byte  flag status2
STAS_LED         EQU    24H         ;1 LED Lock (0000 0CNS)
DBUF0            EQU    25H         ;1
DBUF1            EQU    26H         ;1
COUNT            EQU    27H         ;1
COUNT1           EQU    28H         ;1
BYTESTEP         EQU    29H         ;1
BYTECOUNT        EQU    2AH         ;1
BYTESTEPRIGHT    EQU    2BH         ;1
BYTECOUNTRIGHT   EQU    2CH         ;1
COUNTL           EQU    2DH         ;1
KEYTOBUF         EQU    2EH         ;1
COUNTDIG         EQU    2FH         ;1
ADDSEQ           EQU    30H         ;1
;
CTEMP            EQU    STATUS.0
STOPBIT          EQU    STATUS.1
ARROWLEFT        EQU    STATUS.2
ARROWRIGH        EQU    STATUS.3
ARROWUP          EQU    STATUS.4
ARROWDOWN        EQU    STATUS.5
ENDPROGRAM       EQU    STATUS.6
STARTKEY         EQU    STATUS.7
;
STOPRIGHT        EQU    STATUS0.0
STOPLEFT         EQU    STATUS0.1
STORELEFT        EQU    STATUS0.2
STORERIGHT       EQU    STATUS0.3
STOREUP          EQU    STATUS0.4
STOREDOWN        EQU    STATUS0.5
FREE             EQU    STATUS0.6
INVERTBITS       EQU    STATUS0.7
                 ;
                 ORG     31H
                 ;
IN_BUFF:         DS      1                       ; buffer read in
OUT_BUF:         DS      1                       ; buffer send out
KEY_BUFF:        DS      8                       ; key code 8 byte
DBUF:            DS      1
PLAYBUF:         DS      1
PLAYL:           DS      1
PLAYH:           DS      1
UPCOUNT:         DS      1
DOWNCOUNT:       DS      1
COUNTER_LIMIT:   DS      1
DATASTORE:       DS      1
STACK:           DS      32                      ; 32 Byte Stack
;
SCR_LED          EQU     STAS_LED.0              ; Scroll Lock LED
NUM_LED          EQU     STAS_LED.1              ; Num Lock LED
CAP_LED          EQU     STAS_LED.2              ; Caps Lock LED
INVERTBIT        EQU     STAS_LED.3              ; Invert bit
;
CTL_LFT          EQU     STAS_KEY1.0             ; Ctrl Left Status
CTL_RGT          EQU     STAS_KEY1.1             ; Ctrl Right status
ALT_LFT          EQU     STAS_KEY1.2             ; Alt Left status
ALT_RGT          EQU     STAS_KEY1.3             ; Alt Right status
SHF_LFT          EQU     STAS_KEY1.4             ; Shift Left status
SHF_RGT          EQU     STAS_KEY1.5             ; Shift Right status
CAP_KEY          EQU     STAS_KEY1.6             ; Caps Lock status
NUM_KEY          EQU     STAS_KEY1.7             ; Num Lock status
;
SCR_KEY          EQU     STAS_KEY2.0             ; Scroll Lock status
CTRL_KEY         EQU     STAS_KEY2.1             ; Ctrl key status
ALT_KEY          EQU     STAS_KEY2.2             ; Alt key status
SHF_KEY          EQU     STAS_KEY2.3             ; Shift Key status
UP_KEY           EQU     STAS_KEY2.4             ; Big/Small char status
NUM_STAS         EQU     STAS_KEY2.5             ; Numeric key status
PAR_RD           EQU     STAS_KEY2.6             ; parity buf. of read key
PARITY           EQU     STAS_KEY2.7             ; parity bit buffer
;
DAT_SIGN         EQU     P3.2                    ; Data Signal (INT0) 12
CLK_SIGN         EQU     P3.4                    ; Clock Signal (T0) 14
MODE             EQU     P3.5                    ; Select Display/Program
;
PORTA            EQU     0E0E0H
PORTB            EQU     0E0E1H
PORTC            EQU     0E0E2H
CONT             EQU     0E0E3H
;
                 ORG     0000H                   ; 0000H
                 LJMP    PROGRAM                 ; main program
                 ORG     0003H                   ; 0003H
                 LJMP    INT_READ                ; interupt external 0
;
PROGRAM:         MOV     SP,#STACK               ; Stack 32 Byte
INIT_SR:         MOV     A,#0FDH                 ; Set baud rate 9600
                 MOV     TH1,A
                 MOV     TL1,A
                 MOV     TMOD,#00100000B         ; TIMER1 MODE 1
                 CLR     ES                      ; Disable serial interupt
                 CLR     ET1                     ; Disable timer1 interupt
                 SETB    TR1                     ; Set timer1 control
                 MOV     SCON,#01010000B         ; Serial mode 1
                 ;
                 MOV     STAS_LED,#00H           ; clear all status
                 MOV     STAS_KEY1,#00H
                 MOV     STAS_KEY2,#00H
                 SETB    CLK_SIGN
                 SETB    DAT_SIGN
                 CLR     ARROWLEFT
                 CLR     ARROWRIGH
                 CLR     ARROWUP
                 CLR     ARROWDOWN
                 CLR     ENDPROGRAM
                 CLR     STARTKEY
                 CLR     STORELEFT
                 CLR     STORERIGHT
                 CLR     STOREUP
                 CLR     STOREDOWN
                 MOV     UPCOUNT,#00
                 MOV     DOWNCOUNT,#00
                 ;
;*******************************************************
;*                     DISPLAY BOARD                   *
;*******************************************************
                 LCALL   DELAY
                 ;
                 MOV     DPTR,#CONT
                 MOV     A,#80H
                 MOVX    @DPTR,A
                 ;
                 MOV     BYTESTEP,#08
                 MOV     BYTECOUNT,#1
                 MOV     BYTESTEPRIGHT,#08
                 MOV     COUNTDIG,#00
                 MOV     COUNTER_LIMIT,#00
                 ;
                 MOV     OUT_BUF,#0FFH
                 LCALL   SEND_KEY                ; Reset Key
                 LCALL   READ_KEY                ; ACK
                 LCALL   READ_KEY                ; BAT result
                 MOV     OUT_BUF,#0EDH
                 LCALL   SEND_KEY                ; LED Lock status
                 LCALL   READ_KEY                ; ACK
                 CPL     NUM_LED
                 CPL     NUM_KEY
                 CPL     NUM_STAS
                 MOV     OUT_BUF,STAS_LED        ; Num Lock LED on
                 LCALL   SEND_KEY
                 LCALL   READ_KEY                ; ACK
                 SETB    EX0                     ; enable external interupt0
                 CLR     IT0                     ; interupt0 control by level
                 SETB    EA                      ; Global enable interupt
                 ;
;*********************************************
;*              MAIN   PROGRAM               *
;*********************************************
MAIN:            JNB     MODE,PROGRAMMODE       ;select run/key (1:run,0:key)
                 MOV     PLAYBUF,#0
                 MOV     DPTR,#ACBUF
                 MOV     PLAYH,DPH
                 MOV     PLAYL,DPL
                 LCALL   LOOPWORK
                 SETB    STARTKEY
                 LJMP    MAIN
;*********************************************
;*            KEY BOARD PROGRAM              *
;*********************************************
PROGRAMMODE:     SETB    INVERTBITS
                 JNB     STARTKEY,MAIN3
                 MOV     DPTR,#DATABUF
                 LCALL   READYX
                 LCALL   SUBDISPLAY
                 MOV     ADDSEQ,#0
                 MOV     DPTR,#ACBUF
                 MOV     UPCOUNT,DPH
                 MOV     DOWNCOUNT,DPL
                 CLR     STARTKEY
                 ;
MAIN3:           LCALL   COMMANDDIS
                 MOV     R2,#8                   ;clear keyboard buffer
                 MOV     R0,#KEY_BUFF
CLR_KBUF:        MOV     @R0,#0
                 INC     R0
                 DJNZ    R2,CLR_KBUF
                 ;
;/************************************************************/;
;/*            Scan buffer for detect key press              */;
;/************************************************************/;
LOOP:            MOV     R0,#KEY_BUFF            ;1st buffer
                 CJNE    @R0,#00H,GET_BUF1
                 LCALL   SUBDISPLAY
                 JB      MODE,MAIN
                 SJMP    LOOP
GET_BUF1:        INC     R0                      ;2nd buffer
                 CJNE    @R0,#00H,GET_BUF2
                 LJMP    CODE_1BYTE
GET_BUF2:        INC     R0                      ;3rd buffer
                 CJNE    @R0,#00H,GET_BUF3
                 LJMP    CODE_2BYTE
GET_BUF3:        INC     R0                      ;4th buffer
                 CJNE    @R0,#00H,GET_BUF4
                 LJMP    CODE_3BYTE
GET_BUF4:        INC     R0                      ;5th buffer
                 CJNE    @R0,#00H,GET_BUF5
                 LJMP    CODE_4BYTE
GET_BUF5:        INC     R0                      ;6th buffer
                 CJNE    @R0,#00H,GET_BUF6
                 LJMP    CODE_5BYTE
GET_BUF6:        INC     R0                      ;7th buffer
                 CJNE    @R0,#00H,GET_BUF7
                 LJMP    MAIN
GET_BUF7:        INC     R0                      ;8th buffer
                 CJNE    @R0,#00H,GET_BUF8
                 LJMP    MAIN
GET_BUF8:        LJMP    CODE_8BYTE
;
;/************************************************/;
;/                KEY CODE 1 BYTE                */;
;/************************************************/;
CODE_1BYTE:      MOV     A,KEY_BUFF              ; key code group 1 byte
PRESS_CAP:       CJNE    A,#58H,PRES_NUM         ; Caps Lock key
                 JB      CAP_KEY,OUT_PCAP
                 SETB    CAP_KEY
                 CPL     UP_KEY                  ; Character status
                 CPL     CAP_LED                 ; Caps Lock status
                 MOV     OUT_BUF,#0EDH
                 LCALL   SEND_KEY                ; Lock LED Indicator
                 LCALL   READ_KEY                ; ACK
                 MOV     OUT_BUF,STAS_LED
                 LCALL   SEND_KEY                ; Caps Lock LED on
                 LCALL   READ_KEY                ; ACK
OUT_PCAP:        LJMP    MAIN
                 ;
PRES_NUM:        CJNE    A,#77H,PRES_SCR         ; Num Lock key
                 JB      NUM_KEY,OUT_PNUM
                 SETB    NUM_KEY
                 CPL     NUM_LED                 ; Num Lock status
                 CPL     NUM_STAS
                 MOV     OUT_BUF,#0EDH
                 LCALL   SEND_KEY                ; Lock LED Indicator
                 LCALL   READ_KEY                ; ACK
                 MOV     OUT_BUF,STAS_LED
                 LCALL   SEND_KEY                ; Num Lock LED on
                 LCALL   READ_KEY                ; ACK
OUT_PNUM:        LJMP    MAIN
                 ;
PRES_SCR:        CJNE    A,#7EH,PRES_SHL         ; Scroll Lock key
                 JB      SCR_KEY,OUT_PSCR
                 SETB    SCR_KEY
                 CPL     SCR_LED                 ; Scroll Lock status
                 MOV     OUT_BUF,#0EDH
                 LCALL   SEND_KEY                ; Lock LED Indicator
                 LCALL   READ_KEY                ; ACK
                 MOV     OUT_BUF,STAS_LED
                 LCALL   SEND_KEY                ; Scroll Lock LED on
                 LCALL   READ_KEY                ; ACK
OUT_PSCR:        LJMP    MAIN
                 ;
PRES_SHL:        CJNE    A,#12H,PRES_SHR         ; Shift Left key
                 JB      SHF_LFT,OUT_PSHL
                 SETB    SHF_LFT
                 SETB    SHF_KEY
                 CPL     UP_KEY                  ; Character status
                 CPL     NUM_STAS
OUT_PSHL:        LJMP    MAIN
                 ;
PRES_SHR:        CJNE    A,#59H,PRES_CTL         ; Shift Right key
                 JB      SHF_RGT,OUT_PSHR
                 SETB    SHF_RGT
                 SETB    SHF_KEY
                 CPL     UP_KEY                  ; Character status
                 CPL     NUM_STAS
OUT_PSHR:        LJMP    MAIN
                 ;
PRES_CTL:        CJNE    A,#14H,PRES_ALL         ; Ctrl Left key
                 JB      CTL_LFT,OUT_PCTL
                 SETB    CTL_LFT
                 SETB    CTRL_KEY
OUT_PCTL:        LJMP    MAIN
                 ;
PRES_ALL:        CJNE    A,#11H,PRES_1BYTE       ; Alt Left key
                 JB      ALT_LFT,OUT_PALL
                 SETB    ALT_LFT
                 SETB    ALT_KEY
OUT_PALL:        LJMP    MAIN
;
;/***********************************************/;
;/*         begin decode scan code 1byte        */;
;/***********************************************/;
;
PRES_1BYTE:      CJNE    A,#84H,DECODE1          ;Print Screen Case3
                 ;Press Alt + PrnScrn
                 LJMP    MAIN
                 ;
DECODE1:         JNB     CTRL_KEY,PRESS1
                 SJMP    CTRL                    ;Press Ctrl
PRESS1:          JB      SHF_KEY,SHF_OK
                 SJMP    NOT_SHF                 ;not Ctrl & not Shift
SHF_OK:          LJMP    PRS_SHF
                 ; Press Ctrl key
CTRL:            MOV     A,KEY_BUFF              ;1st byte keyboard buffer
                 MOV     DPTR,#ENG_UP            ;Up character case
                 MOVC    A,@A+DPTR               ;Ascii of key
                 CJNE    A,#5AH,CLEAR1
                 LCALL   CLEARSTORE
CLEAR1:          CJNE    A,#40H,CTRL1            ;Character "@"
CTRL1:           JC      CTRL3                   ;A <  40H
                 CJNE    A,#7EH,CTRL2            ;A => 40H
CTRL2:           JNC     CTRL3                   ;A => 7EH
                 ANL     A,#1FH                  ;A = 40H..7DH
                 MOV     DATASTORE,A
                 LCALL   DIS_1BYTE
                 LCALL   STOREDATA
CTRL3:           LJMP    MAIN
                   ;
                   ;not Ctrl & not Shift
                   ;
NOT_SHF:         MOV     A,KEY_BUFF
                   ;
                   ;Test Numeric detect
                   ;
                 JNB     NUM_STAS,KEY1_END
NOT_NUM1:        MOV     DPTR,#ENG_DN            ; Dn character case
                 MOVC    A,@A+DPTR               ; Ascii of key
                 CJNE    A,#5AH,CLEAR2
                 LCALL   CLEARSTORE
CLEAR2:          JNB     UP_KEY,NOT_SHF3
                 CJNE    A,#61H,NOT_SHF1         ; Character "a"
NOT_SHF1:        JC      NOT_SHF3                ; A < 61H
                 CJNE    A,#7BH,NOT_SHF2         ; Character after "z"
NOT_SHF2:        JNC     NOT_SHF3                ; A => 7BH (61H..7AH)
                 CLR     CY                      ; Character a..z
                 SUBB    A,#20H                  ; adjust to Up Character
NOT_SHF3:        MOV     DATASTORE,A
                 LCALL   DIS_1BYTE
                 LCALL   STOREDATA
                 LJMP    MAIN
                 ;
;***********************************************************************;
                 ;
KEY1_END:        CJNE    A,#69H,KEY4_LFT         ; 1 (End)
                 SETB    ENDPROGRAM
                 LJMP    MAIN
                 ;
KEY4_LFT:        CJNE    A,#6BH,KEY7_HOME        ; 4 (Left)
                 LJMP    MAIN
                 ;
KEY7_HOME:       CJNE    A,#6CH,KEY0_INS         ;Home
                 LJMP    MAIN
                 ;
KEY0_INS:        CJNE    A,#70H,DOT_DEL          ;Insert
                 LJMP    MAIN
                 ;
DOT_DEL:         CJNE    A,#71H,KEY2_DN          ;Del
                 LJMP    MAIN
                 ;
KEY2_DN:         CJNE    A,#72H,KEY5_NUM         ;Dn
                 SETB    ARROWDOWN
                 LJMP    MAIN
                 ;
KEY5_NUM:        CJNE    A,#73H,KEY6_RGT         ;Left
                 SETB    ARROWLEFT
                 LJMP    MAIN
                 ;
KEY6_RGT:        CJNE    A,#74H,KEY8_UP          ;Right
                 SETB    ARROWRIGH
                 LJMP    MAIN
                 ;
KEY8_UP:         CJNE    A,#75H,KEY3_PGDN        ;Up
                 SETB    ARROWUP
                 LJMP    MAIN
                 ;
KEY3_PGDN:       CJNE    A,#7AH,KEY9_PGUP        ;PgPn
                 LJMP    MAIN
                 ;
KEY9_PGUP:       CJNE    A,#7DH,NOT_NUM          ;PgUp
                 LJMP    MAIN
                 ;
NOT_NUM:         JB      SHF_KEY,NOT_NUM2
                 LJMP    NOT_NUM1
STRT_NUM:        LJMP    KEY1_END
                 ;Press Shift key
PRS_SHF:         MOV     A,KEY_BUFF              ; 1st byte keyboard buffer
                 JNB     NUM_STAS,STRT_NUM
NOT_NUM2:        MOV     DPTR,#ENG_UP            ; Up character case
                 MOVC    A,@A+DPTR               ; Ascii of key
                 JB      UP_KEY,PRS_SHF3
                 CJNE    A,#41H,PRS_SHF1         ; Character "A"
PRS_SHF1:        JC      PRS_SHF3                ; A < 41H
                 CJNE    A,#5BH,PRS_SHF2         ; Character after "Z"
PRS_SHF2:        JNC     PRS_SHF3                ; A => 5BH (41H..5AH)
                 CLR     CY                      ; Character A..Z
                 ADD     A,#20H                  ; adjust to Down Character
PRS_SHF3:        MOV     DATASTORE,A
                 LCALL   DIS_1BYTE
                 LCALL   STOREDATA
                 LJMP    MAIN
;/******************************************/;
;/*             KEY CODE 2 BYTE            */;
;/******************************************/;
;
CODE_2BYTE:      MOV     A,KEY_BUFF              ; key code group 2 byte
                 CJNE    A,#0F0H,PRES_2BYTE
                 MOV     A,KEY_BUFF+1            ; Get break code key
RESET_CAP:       CJNE    A,#58H,RESET_NUM        ; release Caps Lock
                 CLR     CAP_KEY
                 LJMP    MAIN
                 ;
RESET_NUM:       CJNE    A,#77H,RESET_SCR        ; release Num Lock
                 CLR     NUM_KEY
                 LJMP    MAIN
                 ;
RESET_SCR:       CJNE    A,#7EH,RESET_SHL        ; release Scroll Lock
                 CLR     SCR_KEY
                 LJMP    MAIN
                 ;
RESET_SHL:       CJNE    A,#12H,RESET_SHR        ; release Shift Left
                 CLR     SHF_LFT
                 CLR     SHF_KEY
                 CPL     UP_KEY
                 CPL     NUM_STAS
                 LJMP    MAIN
                 ;
RESET_SHR:       CJNE    A,#59H,RESET_CTL        ; release Shift Right
                 CLR     SHF_RGT
                 CLR     SHF_KEY
                 CPL     UP_KEY
                 CPL     NUM_STAS
                 LJMP    MAIN
                 ;
RESET_CTL:       CJNE    A,#14H,RESET_ALL        ; release Ctrl Left
                 CLR     CTL_LFT
                 CLR     CTRL_KEY
                 LJMP    MAIN
                 ;
RESET_ALL:       CJNE    A,#11H,OUT_RALL         ; release Alt Left
                 CLR     ALT_LFT
                 CLR     ALT_KEY
OUT_RALL:        LJMP    MAIN
                 ;
;/********************************/;
;/* begin decode scan code 2byte */;
;/********************************/;
;
PRES_2BYTE:  CJNE    A,#0E0H,OUT_RALR            ; Press 2byte
             MOV     A,KEY_BUFF+1                ; Get Make code key
PRES_ALR:    CJNE    A,#11H,PRES_CTR             ; Alt Right
             JB      ALT_RGT,OUT_RALR
             SETB    ALT_RGT
             SETB    ALT_KEY
OUT_RALR:    LJMP    MAIN
             ;
PRES_CTR:    CJNE    A,#14H,PRES_ENT             ; Ctrl Right
             JB      CTL_RGT,OUT_RCTR
             SETB    CTL_RGT
             SETB    CTRL_KEY
OUT_RCTR:    LJMP    MAIN
             ;
PRES_ENT:    CJNE    A,#5AH,INSERT_1             ; Enter Right
             MOV     A,#0DH
             LJMP    MAIN
               ;
             ;Not Num Lock + Not Shift
             ;Num Lock On + Press Shift
INSERT_1:    CJNE    A,#70H,DELETE_1             ; Insert (Case1)
             LJMP    MAIN
             ;
DELETE_1:    CJNE    A,#71H,HOME_1               ; Delete (Case1)
             LJMP    MAIN
             ;
HOME_1:     CJNE    A,#6CH,END_1                 ; Home (Case1)
            LJMP    MAIN
            ;
END_1:      CJNE    A,#69H,PGUP_1                ; End (Case1)
            SETB    ENDPROGRAM
            LJMP    MAIN
            ;
PGUP_1:     CJNE    A,#7DH,PGDN_1                ; Page Up (Case1)
            LJMP    MAIN
            ;
PGDN_1:     CJNE    A,#7AH,LEFT_1                ; Page Down (Case1)
            LJMP    MAIN
            ;
LEFT_1:     CJNE    A,#6BH,RIGHT_1               ; Left arrow (Case1)
            SETB    ARROWLEFT
            LJMP    MAIN
            ;
RIGHT_1:    CJNE    A,#74H,UP_1                 ; Right arrow (Case1)
            SETB    ARROWRIGH
            LJMP    MAIN
            ;
UP_1:       CJNE    A,#75H,DOWN_1               ; Up arrow (Case1)
            SETB    ARROWUP
            LJMP    MAIN
            ;
DOWN_1:     CJNE    A,#72H,SLASE_1              ; Down arrow (Case1)
            SETB    ARROWDOWN
            LJMP    MAIN
            ;
SLASE_1:    CJNE    A,#4AH,PRNSCRN_2            ; Slase "/" (Case1)
            LJMP    MAIN
               ;
            ;Press Shift + PrnScrn
            ;Press Ctrl + PrnScrn
PRNSCRN_2:  CJNE    A,#7CH,OUT_2BYTE            ; Print Screen (case2)
OUT_2BYTE:  LJMP    MAIN
;
;/******************************************/;
;/*            KEY CODE 3 BYTE             */;
;/******************************************/;
CODE_3BYTE: MOV     A,KEY_BUFF               ; key code group 3 byte
            CJNE    A,#0E0H,OUT_3BYTE
            MOV     A,KEY_BUFF+1
            CJNE    A,#0F0H,OUT_3BYTE
            MOV     A,KEY_BUFF+2
RESET_ALR:  CJNE    A,#11H,RESET_CTR         ; release Alt Right
            CLR     ALT_RGT
            CLR     ALT_KEY
            LJMP    MAIN
            ;
RESET_CTR:  CJNE    A,#14H,OUT_3BYTE         ; release Ctrl Right
            CLR     CTL_RGT
            CLR     CTRL_KEY
OUT_3BYTE:  LJMP    MAIN
;
;/*****************************************/;
;/*             KEY CODE 4 BYTE           */;
;/*****************************************/;
CODE_4BYTE: MOV     A,KEY_BUFF
            ;key code 4 byte
            CJNE    A,#0E0H,OUT_4BYTE
            MOV     A,KEY_BUFF+1
            CJNE    A,#12H,PRES_SLAS2
            MOV     A,KEY_BUFF+2
            CJNE    A,#0E0H,OUT_4BYTE
            SJMP    PRES_4BYTE
OUT_4BYTE:  LJMP    MAIN
            ;
PRES_SLAS2: CJNE    A,#0F0H,OUT_4BYTE
            MOV     A,KEY_BUFF+2
            CJNE    A,#12H,OUT_4BYTE
            MOV     A,KEY_BUFF+3
            CJNE    A,#4AH,OUT_4BYTE        ; Slase "/" (Case2)
            LJMP    MAIN
            ;Num Lock On + Not Shift
PRES_4BYTE: MOV     A,KEY_BUFF+3
INSERT_3:   CJNE    A,#70H,DELETE_3         ; Insert (Case3)
            LJMP    MAIN
            ;
DELETE_3:   CJNE    A,#71H,HOME_3           ; Delete (Case3)
            LJMP    MAIN
            ;
HOME_3:     CJNE    A,#6CH,END_3            ; Home (Case3)
            LJMP    MAIN
            ;
END_3:      CJNE    A,#69H,PGUP_3           ; End (Case3)
            SETB    ENDPROGRAM
            LJMP    MAIN
            ;
PGUP_3:     CJNE    A,#7DH,PGDN_3           ; Page Up (Case3)
            LJMP    MAIN
            ;
PGDN_3:     CJNE    A,#7AH,LEFT_3           ; Page Down (Case3)
            LJMP    MAIN
            ;
LEFT_3:     CJNE    A,#6BH,RIGHT_3          ; Left arrow (Case3)
            SETB    ARROWLEFT
            LJMP    MAIN
            ;
RIGHT_3:    CJNE    A,#74H,UP_3             ; Right arrow (Case3)
            SETB    ARROWRIGH
            LJMP    MAIN
            ;
UP_3:       CJNE    A,#75H,DOWN_3
            ;Up arrow (Case3)
            SETB    ARROWUP
            LJMP    MAIN
            ;
DOWN_3:     CJNE    A,#72H,PRNSCRN_1        ; Down arrow (Case3)
            SETB    ARROWDOWN
            LJMP    MAIN
            ;
            ;Ctrl,Alt,Shift not Press
PRNSCRN_1:  CJNE    A,#7CH,OUT_PRN1         ; Print Screen (case1)
OUT_PRN1:   LJMP    MAIN
;
;/*******************************************/;
;/*            KEY CODE 5 BYTE              */;
;/*******************************************/;
;
CODE_5BYTE: MOV     A,KEY_BUFF              ; key code group 3 byte
            CJNE    A,#0E0H,OUT_5BYTE
            MOV     A,KEY_BUFF+1
            CJNE    A,#0F0H,PAUSE_2
            MOV     A,KEY_BUFF+2
            CJNE    A,#12H,OUT_5BYTE
            MOV     A,KEY_BUFF+3
            CJNE    A,#0E0H,OUT_5BYTE
            SJMP    PRES_5BYTE
OUT_5BYTE:  LJMP    MAIN
            ;
PAUSE_2:    CJNE    A,#7EH,OUT_5BYTE         ;Pause (case2)
            MOV     A,KEY_BUFF+2
            CJNE    A,#0E0H,OUT_5BYTE
            MOV     A,KEY_BUFF+3
            CJNE    A,#0F0H,OUT_5BYTE
            MOV     A,KEY_BUFF+4
            CJNE    A,#7EH,OUT_5BYTE
            ;Press Ctrl + Pause
            LJMP    MAIN
            ;
            ;Num Lock off + Press Shift
PRES_5BYTE: MOV     A,KEY_BUFF+4H
INSERT_5:   CJNE    A,#70H,DELETE_5         ; Insert (Case2)
            LJMP    MAIN
            ;
DELETE_5:   CJNE    A,#71H,HOME_5           ; Delete (Case2)
            LJMP    MAIN
            ;
HOME_5:     CJNE    A,#6CH,END_5            ; Home (Case2)
            LJMP    MAIN
            ;
END_5:      CJNE    A,#69H,PGUP_5           ; End (Case2)
            SETB    ENDPROGRAM
            LJMP    MAIN
            ;
PGUP_5:     CJNE    A,#7DH,PGDN_5           ; Page Up (Case2)
            SETB    ARROWUP
            LJMP    MAIN
            ;
PGDN_5:     CJNE    A,#7AH,LEFT_5           ; Page Down (Case2)
            SETB    ARROWDOWN
            LJMP    MAIN
            ;
LEFT_5:     CJNE    A,#6BH,RIGHT_5          ; Left arrow (Case2)
            SETB    ARROWLEFT
            LJMP    MAIN
            ;
RIGHT_5:    CJNE    A,#74H,UP_5              ; Right arrow (Case2)
            SETB    ARROWRIGH
            LJMP    MAIN
            ;
UP_5:       CJNE    A,#75H,DOWN_5            ; Up arrow (Case2)
            SETB    ARROWUP
            LJMP    MAIN
            ;
DOWN_5:     CJNE    A,#72H,OUT_DN5           ; Down arrow (Case2)
            SETB    ARROWDOWN
OUT_DN5:    LJMP    MAIN
            ;
;/*********************************************/;
;/*             KEY CODE 8 BYTE               */;
;/*********************************************/;
;
CODE_8BYTE: MOV     A,KEY_BUFF                  ; key code group 3 byte
            CJNE    A,#0E1H,OUT_8BYTE
            MOV     A,KEY_BUFF+1
            CJNE    A,#14H,OUT_8BYTE
            MOV     A,KEY_BUFF+2
            CJNE    A,#77H,OUT_8BYTE
            MOV     A,KEY_BUFF+3
            CJNE    A,#0E1H,OUT_8BYTE
            MOV     A,KEY_BUFF+4
            CJNE    A,#0F0H,OUT_8BYTE
            MOV     A,KEY_BUFF+5
            CJNE    A,#14H,OUT_8BYTE
            MOV     A,KEY_BUFF+6
            CJNE    A,#0F0H,OUT_8BYTE
            MOV     A,KEY_BUFF+7
            CJNE    A,#77H,OUT_8BYTE
            ;Ctrl Key not Press
OUT_8BYTE:  LJMP    MAIN
;
;/******************************/;
;/* Receive Data from Keyboard */;
;/* Output  =  Buffer : IN_BUF */;
;/******************************/;
;
READ_KEY:   CLR     EA                      ; disable interupt
            PUSH    PSW
            SETB    RS0                     ; select bank1
            MOV     R0,#KEY_BUFF
            ;
STRT_RXD:   LCALL   CLK_DN                  ; start bit
            LCALL   CLK_UP
            ;
            CLR     A
            MOV     R1,A
            MOV     R2,#8                   ; 8 bit data
RXD_KEY:    LCALL   CLK_DN                  ; Read data D0..D7
            MOV     A,P1
            MOV     C,DAT_SIGN              ; Read data from keyboard
            MOV     A,R1
            RRC     A
            MOV     R1,A
            LCALL   CLK_UP
            DJNZ    R2,RXD_KEY
            MOV     IN_BUFF,R1
            ;
            LCALL   CLK_DN
            MOV     A,P1
            MOV     PARITY,DAT_SIGN         ; Parity bit
            LCALL   CLK_UP
            ;
            LCALL   CLK_DN                  ; stop bit
            LCALL   CLK_UP
            LCALL   DATA_UP
            MOV     @R0,IN_BUFF
            ;
TIME_OUT:   SETB    DAT_SIGN
            MOV     R3,#05H
TIME1:      MOV     R2,#0FFH
TIME2:      JNB     DAT_SIGN,NEW_BYTE
            DJNZ    R2,TIME2
            DJNZ    R3,TIME1
            AJMP    END_IN
NEW_BYTE:   INC     R0                      ; Next buffer
            AJMP    STRT_RXD
            ;
END_IN:     POP     PSW
            SETB    EA                      ; enable interupt
            RET
            ;
;/*********************************/;
;/* Send Command/Data to Keyboard */;
;/* Input  =  Buffer : OUT_BUF    */;
;/*********************************/;
;
SEND_KEY:   CLR     EA                      ; disable interupt
            PUSH    PSW
            SETB    RS1                     ; select bank2
            ;
            MOV     A,OUT_BUF               ; prepared parity
            JNB     P,EVEN
            CLR     CY                      ; parity odd
            SJMP    STRT_TXD
EVEN:       SETB    CY                      ; parity even
            ;
STRT_TXD:   MOV     R2,#9                   ; 8 bit data + Parity
            CLR     DAT_SIGN                ; start bit
            LCALL   CLK_DN
            ;
TXD_KEY:    RRC     A                       ; Shift Out D0..D7,Parity Odd
            MOV     DAT_SIGN,C
            LCALL   CLK_UP
            LCALL   CLK_DN
            DJNZ    R2,TXD_KEY
            ;
            SETB    DAT_SIGN                ; Stop bit
            LCALL   CLK_UP
            LCALL   CLK_DN
            LCALL   CLK_UP
            LCALL   DATA_UP
            ;
            POP     PSW
            SETB    EA                      ; enable interupt
            RET
            ;
;/*******************************/;
;/* EXTERNAL INTERUPT 1 SERVICE */;
;/*     USES REGISTER BANK 3    */;
;/* INPUT  : DATA FROM KEYBOARD */;
;/* OUTPUT : SAVE_BUF (8 byte)  */;
;/*******************************/;
;
INT_READ:   LCALL   READ_KEY
            RETI
;
;/**************************/;
;/* Wait Rising edge Clock */;
;/**************************/;
;
CLK_UP:     SETB    CLK_SIGN                ; wait rising edge
            JNB     CLK_SIGN,$
            RET
;
;/***************************/;
;/* Wait Falling edge Clock */;
;/***************************/;
;
CLK_DN:     SETB    CLK_SIGN                ; wait falling edge
            JB      CLK_SIGN,$
            RET
;
;/*************************/;
;/* Wait Rising edge Data */;
;/*************************/;
;
DATA_UP:    SETB    DAT_SIGN
            JNB     DAT_SIGN,$
            RET
;
;/**************************/;
;/* Wait Falling edge Data */;
;/**************************/;
;
DATA_DN:    SETB    DAT_SIGN
            JB      DAT_SIGN,$
            RET
;
            ; Up String (Shift on only)
ENG_UP:     DB      00H,00H,00H,00H,00H
            DB      00H,00H,00H,00H,00H
            DB      00H,00H,00H,09H,7EH
            DB      00H,00H,00H,00H,00H
            DB      00H,51H,21H,00H,00H
            DB      00H,5AH,53H,41H,57H
            DB      40H,00H,00H,43H,58H
            DB      44H,45H,24H,23H,00H
            DB      00H,20H,56H,46H,54H
            DB      52H,25H,00H,00H,4EH
            DB      42H,48H,47H,59H,5EH
            DB      00H,00H,00H,4DH,4AH
            DB      55H,26H,2AH,00H,00H
            DB      3CH,4BH,49H,4FH,29H
            DB      28H,00H,00H,3EH,3FH
            DB      4CH,3AH,50H,5FH,00H
            DB      00H,00H,22H,00H,7BH
            DB      2BH,00H,00H,00H,00H
            DB      0DH,7DH,00H,7CH,00H
            DB      00H,00H,00H,00H,00H
            ;
            DB      00H,00H,08H,00H,00H
            DB      31H,00H,34H,37H,00H
            DB      00H,00H,30H,2EH,32H
            DB      35H,36H,38H,1BH,00H
            DB      00H,2BH,33H,00H,2AH
            DB      39H,00H,00H,00H,00H
            DB      00H,00H
;
            ; Down String (Shift off only)
ENG_DN:     DB      00H,00H,00H,00H,00H
            DB      00H,00H,00H,00H,00H
            DB      00H,00H,00H,09H,60H
            DB      00H,00H,00H,00H,00H
            DB      00H,71H,31H,00H,00H
            DB      00H,7AH,73H,61H,77H
            DB      32H,00H,00H,63H,78H
            DB      64H,65H,34H,33H,00H
            DB      00H,20H,76H,66H,74H
            DB      72H,35H,00H,00H,6EH
            DB      62H,68H,67H,79H,36H
            DB      00H,00H,00H,6DH,6AH
            DB      75H,37H,38H,00H,00H
            DB      2CH,6BH,69H,6FH,30H
            DB      39H,00H,00H,2EH,2FH
            DB      6CH,3BH,70H,2DH,00H
            DB      00H,00H,27H,00H,5BH
            DB      3DH,00H,00H,00H,00H
            DB      0DH,5DH,00H,5CH,00H
            DB      00H,00H,00H,00H,00H
            ;
            DB      00H,00H,08H,00H,00H
            DB      31H,00H,34H,37H,00H
            DB      00H,00H,30H,2EH,32H
            DB      35H,36H,38H,1BH,00H
            DB      00H,2BH,33H,2DH,2AH
            DB      39H,00H,00H,00H,00H
            DB      00H,00H
                ;
;***************************************************
;*          PROGRAM  SEND DATA TO BUFFER           *
;* RECIVE  DATA FROM  KEYBOARD  AND SEND TO BUFFER *
;* data in ACC                                     *
;***************************************************
STOREDATA:       JNB    STOREUP,STOREDATA1     ;flow up
                 MOV    DPH,UPCOUNT
                 MOV    DPL,DOWNCOUNT
                 MOV    A,DATASTORE
                 MOVX   @DPTR,A                ;save data to buffer
                 INC    DPTR
                 MOV    UPCOUNT,DPH
                 MOV    DOWNCOUNT,DPL
                 LJMP   STOREOUT               ;end of save data in buffer
                 ;
STOREDATA1:      JNB    STOREDOWN,STOREDATA2   ;flow down
                 MOV    DPH,UPCOUNT
                 MOV    DPL,DOWNCOUNT
                 MOV    A,DATASTORE
                 MOVX   @DPTR,A                ;save data to buffer
                 INC    DPTR
                 MOV    UPCOUNT,DPH
                 MOV    DOWNCOUNT,DPL
                 LJMP   STOREOUT               ;end of save data in buffer
                 ;
STOREDATA2:      JNB    STORELEFT,STOREDATA3   ;flow left
                 MOV    DPH,UPCOUNT
                 MOV    DPL,DOWNCOUNT
                 MOV    A,DATASTORE
                 MOVX   @DPTR,A                ;save data to buffer
                 INC    DPTR
                 MOV    UPCOUNT,DPH
                 MOV    DOWNCOUNT,DPL
                 LJMP   STOREOUT               ;end of save data to buffer
                 ;
STOREDATA3:      JNB    STORERIGHT,STOREOUT    ;flow right
                 MOV    DPH,UPCOUNT
                 MOV    DPL,DOWNCOUNT
                 MOV    A,DATASTORE
                 MOVX   @DPTR,A                ;save data to buffer
                 INC    DPTR
                 MOV    UPCOUNT,DPH
                 MOV    DOWNCOUNT,DPL
STOREOUT:        RET
                 ;
CLEARSTORE:      CLR    STOREUP
                 CLR    STOREDOWN
                 CLR    STORERIGHT
                 CLR    STORELEFT
                 RET
                 ;
;************************************************
;*             COMMAND  KEY  PROGRAM            *
;************************************************
COMMANDDIS:      JNB    ARROWLEFT,COMMANDDIS1       ;shift left symbol
                 MOV    DPTR,#SEQBUF
                 MOV    A,ADDSEQ
                 ADD    A,DPL
                 MOV    DPL,A
                 MOV    A,DPH
                 ADDC   A,#0
                 MOV    DPH,A
                 MOV    A,#03H
                 MOVX   @DPTR,A
                 INC    ADDSEQ
                 MOV    DPTR,#CLRBUF                ;clear display
                 LCALL  READYX
                 MOV    A,#3CH                      ;data of "<"
                 LCALL  WRITE_COMMAND
                 CLR    ARROWLEFT                   ;used for left finish
                 LCALL  CLEARSTORE
                 SETB   STORELEFT
                 MOV    COUNTDIG,#0
                 LJMP   COMMANDOUT
                 ;
COMMANDDIS1:     JNB    ARROWRIGH,COMMANDDIS2       ;shift right symbol
                 MOV    DPTR,#SEQBUF
                 MOV    A,ADDSEQ
                 ADD    A,DPL
                 MOV    DPL,A
                 MOV    A,DPH
                 ADDC   A,#0
                 MOV    DPH,A
                 MOV    A,#04H
                 MOVX   @DPTR,A
                 INC    ADDSEQ
                 MOV    DPTR,#CLRBUF                ;clear display
                 LCALL  READYX
                 MOV    A,#3EH                      ;data of ">"
                 LCALL  WRITE_COMMAND
                 CLR    ARROWRIGH                   ;used for right finish
                 LCALL  CLEARSTORE
                 SETB   STORERIGHT
                 MOV    COUNTDIG,#0
                 LJMP   COMMANDOUT
;                 ;
COMMANDDIS2:     JNB    ARROWUP,COMMANDDIS3         ;shift up symbol
                 MOV    DPTR,#SEQBUF
                 MOV    A,ADDSEQ
                 ADD    A,DPL
                 MOV    DPL,A
                 MOV    A,DPH
                 ADDC   A,#0
                 MOV    DPH,A
                 MOV    A,#01H
                 MOVX   @DPTR,A
                 INC    ADDSEQ
                 MOV    DPTR,#CLRBUF                ;clear display
                 LCALL  READYX
                 MOV    A,#5EH                      ;data of up
                 LCALL  WRITE_COMMAND
                 CLR    ARROWUP                     ;used for up finish
                 LCALL  CLEARSTORE
                 SETB   STOREUP
                 MOV    COUNTDIG,#0
                 LJMP   COMMANDOUT
                 ;
COMMANDDIS3:     JNB    ARROWDOWN,COMMANDDIS4       ;shift down symbol
                 MOV    DPTR,#SEQBUF
                 MOV    A,ADDSEQ
                 ADD    A,DPL
                 MOV    DPL,A
                 MOV    A,DPH
                 ADDC   A,#0
                 MOV    DPH,A
                 MOV    A,#02H
                 MOVX   @DPTR,A
                 INC    ADDSEQ
                 MOV    DPTR,#CLRBUF                ;clear display
                 LCALL  READYX
                 MOV    A,#76H                      ;data of down
                 LCALL  WRITE_COMMAND
                 CLR    ARROWDOWN                   ;used for down finish
                 LCALL  CLEARSTORE
                 SETB   STOREDOWN
                 MOV    COUNTDIG,#0
                 LJMP   COMMANDOUT
                 ;
COMMANDDIS4:     JNB
                 ;
COMMANDDIS4:     JNB    ENDPROGRAM,COMMANDOUT       ;end of command
                 MOV    DPTR,#SEQBUF
                 MOV    A,ADDSEQ
                 ADD    A,DPL
                 MOV    DPL,A
                 MOV    A,DPH
                 ADDC   A,#0
                 MOV    DPH,A
                 MOV    A,#0DH
                 MOVX   @DPTR,A
                 MOV    DPTR,#ENDDIS
                 LCALL  READYX
                 CLR    ENDPROGRAM
COMMANDOUT:      RET                                 ;finish of display command
                 ;
DATABUF:         DB     ' PROGRAMS ',0DH
CLRBUF:          DB     '          ',0DH
ENDDIS:          DB     'END       ',0DH
DELDIS:          DB     'DELAY     ',0DH
CLRDIS:          DB     'CLR_DIS   ',0DH
CLRALL:          DB     'CLR_ALL   ',0DH
                 ;
;*****************************************************************
;*                        DISPLAY 1 BYTE                         *
;*****************************************************************
DIS_1BYTE:       LCALL  WRITE_CHAR
                 RET
;*****************************************************************
;*                        SCAN  DISPLAY                          *
;*                        R2 = COLUM                             *
;*                        R4 = ROW                               *
;*****************************************************************
SUBDISPLAY:     PUSH    02
                PUSH    04
                LCALL   CLR_DIS                ;clr display
                MOV     A,#00H                 ;tric 74123
                SETB    ACC.7
                MOV     DPTR,#PORTB
                MOVX    @DPTR,A
                ;
                MOV     R4,#00H                ;start row 0
ROWINC:         MOV     R2,#00H                ;start colum 0
                MOV     DPTR,#DISBUF
                MOV     A,R4
                ADD     A,DPL
                MOV     DPL,A
                MOV     A,#00
                ADDC    A,DPH
                PUSH    DPH
                PUSH    DPL
COLINC:         MOV     DPTR,#PORTB
                MOV     A,R2
                ANL     A,#0FH
                SETB    ACC.7
                MOVX    @DPTR,A
                ;
                POP     DPL
                POP     DPH
                PUSH    DPH
                PUSH    DPL
                MOV     A,R2
                MOV     B,#8
                MUL     AB
                ADD     A,DPL
                MOV     DPL,A
                MOV     A,B
                ADDC    A,DPH
                MOV     DPH,A
                MOVX    A,@DPTR
                ;
                MOV     DPTR,#PORTA
                MOVX    @DPTR,A                ;send data to portA
                ;
                MOV    A,R2
                ANL    A,#0FH
                CJNE   A,#09H,INCCOL
                POP    DPL
                POP    DPH
                MOV    DPTR,#PORTB
                MOV    A,#8FH
                MOVX   @DPTR,A
                ;scan  row
                MOV    A,R4
                SWAP   A
                ANL    A,#0F0H
                CLR    ACC.7
                MOVX   @DPTR,A
                LCALL  ROW_DLY
                LCALL  CLR_DIS
                MOV    A,R4
                ANL    A,#0FH
                CJNE   A,#07H,INCROW
                LJMP   OUTDISPLAY
INCCOL:         INC    R2
                LJMP   COLINC
INCROW:         INC    R4
                LJMP   ROWINC
OUTDISPLAY:     POP    04
                POP    02
                RET
;
;***************************************
;*              DELAY ROW              *
;***************************************
ROW_DLY:        PUSH   ACC
                PUSH   B
                MOV    B,#10
RD1:            DEC    B
                MOV    A,B
                JNZ    RD1
                POP    B
                POP    ACC
                RET
;
;**************************************
;*          CLEAR  DISPLAY            *
;*  SEND DATA 00 TO PORT A            *
;*  COLLUM = R3                       *
;**************************************
CLR_DIS:        MOV    R3,#00H               ;start collum  0
COLINC_A:       MOV    DPTR,#PORTB
                MOV    A,R3
                SETB   ACC.7                 ;disable row
                MOVX   @DPTR,A               ;select colum
                MOV    DPTR,#PORTA           ;data port
                MOV    A,#00H
                MOVX   @DPTR,A               ;send 00 to data port
                MOV    A,R3
                ANL    A,#0FH
                CJNE   A,#09H,INCCOL_A       ;collum = 9
                MOV    DPTR,#PORTB
                MOV    A,#8FH                ;set clk 145 = H
                MOVX   @DPTR,A
                LJMP   CLROUT
INCCOL_A:       INC    R3
                LJMP   COLINC_A
CLROUT:         RET                          ;end
;
;*************************************
;*          CLEAR  DISPLAY           *
;*************************************
CLEARDISPLAY:   MOV   R7,#10
                MOV   DPTR,#DISBUF
CLRSCR:         MOV   A,#30H
                MOVX  @DPTR,A
                INC   DPTR
                DJNZ  R7,CLRSCR
                RET
                ;
;*************************************
;*     Write character to display    *
;* INPUT    : R2=Position on display *
;*            ACC=Character          *
;*************************************
WRITE_CHAR:     PUSH   ACC
                MOV    A,COUNTDIG
                CJNE   A,#10,CH_CHAR
CH_CHAR:        JB     CY,CH_CHAR1
                MOV    COUNTDIG,#9
                LCALL  SLISELEFT
CH_CHAR1:       MOV    DPTR,#FONT
                CLR    CY
                POP    ACC
                SUBB   A,#20H        ;Because font start at ASCII 20h
                MOV    B,#08
                MUL    AB
                ADD    A,DPL         ;Low byte of FONT table
                MOV    DPL,A
                MOV    A,B
                ADDC   A,DPH
                MOV    DPH,A
                PUSH   DPH
                PUSH   DPL
                ;
                MOV    DPTR,#DISBUF
                MOV    A,COUNTDIG
                MOV    B,#8
                MUL    AB
                ADD    A,DPL
                MOV    DPL,A
                MOV    A,B
                ADDC   A,DPH
                MOV    DPH,A
                MOV    DBUF1,DPH
                MOV    DBUF0,DPL
                POP    DPL
                POP    DPH
                ;
                MOV   R4,#8           ;8 row
FAC1:           MOV   A,#00
                MOVC  A,@A+DPTR       ;Get flont  1st of flont
                PUSH  DPH
                PUSH  DPL
                MOV   DPH,DBUF1       ;disbuf high address
                MOV   DPL,DBUF0       ;disbuf low address
                MOVX  @DPTR,A         ;Fill  display buffer
                INC   DPTR
                MOV   DBUF1,DPH
                MOV   DBUF0,DPL
                POP   DPL             ;flont low  address
                POP   DPH             ;flont high  address
                INC   DPTR            ;inc flont address
                DJNZ  R4,FAC1
                INC   COUNTDIG
                RET
                ;
;*************************************
;*     Write character to display    *
;* INPUT    : R2=Position on display *
;*            ACC=Character          *
;*************************************
WRITE_COMMAND:  MOV    DPTR,#FONT
                CLR    CY
                SUBB   A,#20H             ;Because font start at ASCII 20h
                MOV    B,#08
                MUL    AB
                ADD    A,DPL              ;Low byte of FONT table
                MOV    DPL,A
                MOV    A,B
                ADDC   A,DPH
                MOV    DPH,A
                PUSH   DPH
                PUSH   DPL
                ;
                MOV    DPTR,#DISBUF
                MOV    A,#0
                MOV    B,#8
                MUL    AB
                ADD    A,DPL
                MOV    DPL,A
                MOV    A,B
                ADDC   A,DPH
                MOV    DPH,A
                MOV    DBUF1,DPH
                MOV    DBUF0,DPL
                POP    DPL
                POP    DPH
                ;
                MOV    R4,#8           ;8 row
FACOMMAND:      MOV    A,#00
                MOVC   A,@A+DPTR       ;Get flont  1st of flont
                PUSH   DPH
                PUSH   DPL
                MOV    DPH,DBUF1       ;disbuf high address
                MOV    DPL,DBUF0       ;disbuf low address
                MOVX   @DPTR,A         ;Fill  display buffer
                INC    DPTR
                MOV    DBUF1,DPH
                MOV    DBUF0,DPL
                POP    DPL             ;flont low  address
                POP    DPH             ;flont high  address
                INC    DPTR            ;inc flont address
                DJNZ   R4,FACOMMAND
                RET
                ;
;*****************************************
;*     SHIFT DATA FROM RIGH TO LEFT      *
;*****************************************
SLISELEFT:      MOV    R4,#0
SLLEFT2:        MOV    R5,#0
                MOV    DPTR,#DISBUF
                MOV    A,DPL
                ADD    A,R4
                MOV    DPL,A
                MOV    A,DPH
                ADDC   A,#0
                MOV    DPH,A
SLLEFT1:        PUSH   DPH
                PUSH   DPL
                MOV    A,DPL
                ADD    A,#8
                MOV    DPL,A
                MOV    A,DPH
                ADDC   A,B
                MOV    DPH,A
                MOVX   A,@DPTR             ;get data provide shift left
                MOV    R1,DPH
                MOV    R0,DPL
                POP    DPL
                POP    DPH
                MOVX   @DPTR,A             ;put data
                MOV    DPH,R1
                MOV    DPL,R0
                MOV    A,R5
                INC    R5
                CJNE   A,#10,SLLEFT1
                MOV    A,R4
                INC    R4
                CJNE   A,#7,SLLEFT2
                RET
                ;
;*************************************
;*     Write character to display    *
;* INPUT    : R2=Position on display *
;*            ACC=Character          *
;*************************************
                ;
WRITE_CHR:      MOV    DPTR,#FONT
                CLR    CY
                SUBB   A,#20H        ;Because font start at ASCII 20h
                MOV    B,#08
                MUL    AB
                ADD    A,DPL         ;Low byte of FONT table
                MOV    DPL,A
                MOV    A,B
                ADDC   A,DPH
                MOV    DPH,A
                PUSH   DPH
                PUSH   DPL
                ;
                MOV    DPTR,#PDISBUF
                MOV    A,R3
                MOV    B,#8
                MUL    AB
                ADD    A,DPL
                MOV    DPL,A
                MOV    A,B
                ADDC   A,DPH
                MOV    DPH,A
                MOV    DBUF1,DPH
                MOV    DBUF0,DPL
                POP    DPL
                POP    DPH
                ;
                MOV   R4,#8                  ;8 row
FC1:            MOV   A,#00
                MOVC  A,@A+DPTR              ;Get flont  1st of flont
                PUSH  DPH
                PUSH  DPL
                MOV   DPH,DBUF1              ;disbuf high address
                MOV   DPL,DBUF0              ;disbuf low address
                JB    INVERTBITS,NOTINVERT
                CPL   A
NOTINVERT:      MOVX  @DPTR,A                ;Fill  display buffer
                INC   DPTR
                MOV   DBUF1,DPH
                MOV   DBUF0,DPL
                POP   DPL                    ;flont low  address
                POP   DPH                    ;flont high  address
                INC   DPTR                   ;inc flont address
                DJNZ  R4,FC1
                RET
                 ;
;********************************************
;*            PRINT                         *
;* Function : Get character string pointed  *
;*            by DPTR and convert to font   *
;*            and put on display buffer     *
;* Input    : DPTR = Address of string      *
;* Note     : end of string by character 0D *
;********************************************
;
PRINT:      MOV    R3,#0
            MOV    PLAYBUF,#1
G1:         MOVX   A,@DPTR
            CJNE   A,#0DH,PROCESS
            MOV    R2,#11
G2:         MOV    A,#20H
            PUSH   DPH
            PUSH   DPL
            LCALL  WRITE_CHR
            INC    R3
            POP    DPL
            POP    DPH
            DJNZ   R2,G2
            SJMP   OUT_MOV
PROCESS:    INC    PLAYBUF
            INC    DPTR
            PUSH   DPH
            PUSH   DPL
            LCALL  WRITE_CHR
            INC    R3                  ;counter of data
            POP    DPL
            POP    DPH
            SJMP   G1                  ;Until meet Ascii 1Ah
OUT_MOV:    RET
            ;
;********************************************
;*            PRINT                         *
;* Function : Get character string pointed  *
;*            by DPTR and convert to font   *
;*            and put on display buffer     *
;* Input    : DPTR = Address of string      *
;* Note     : end of string by character 0D *
;********************************************
;
PRINTX:     MOV    R3,#0
GX1:        MOV    A,R3
            MOVC   A,@A+DPTR           ;Get Ascii
            CJNE   A,#0DH,PROCESSX
            MOV    R2,#11
GX2:        MOV    A,#20H
            PUSH   DPH
            PUSH   DPL
            LCALL  WRITE_CHR
            INC    R3
            POP    DPL
            POP    DPH
            DJNZ   R2,GX2
            SJMP   OUT_MOVX
PROCESSX:   PUSH   DPH
            PUSH   DPL
            LCALL  WRITE_CHR
            INC    R3                  ;counter of data
            POP    DPL
            POP    DPH
            SJMP   GX1                  ;Until meet Ascii 1Ah
OUT_MOVX:   RET
            ;
;**********************************************
;*            PROGRAM  SHIFT RIGHT            *
;**********************************************
SH_RIGHT:        MOV    R2,#8
                 MOV    DPTR,#DISBUF-8
SH_R1:           MOV    A,#00H
                 MOVX   @DPTR,A
                 INC    DPTR
                 DJNZ   R2,SH_R1
                 MOV    BYTESTEPRIGHT,#08H      ;define for 8 byte in 1 digit
                 CLR    STOPRIGHT               ;on shift right
                 MOV    DPH,PLAYH
                 MOV    DPL,PLAYL
                 MOV    A,PLAYBUF
                 ADD    A,DPL
                 MOV    DPL,A
                 MOV    A,DPH
                 ADDC   A,#0
                 MOV    DPH,A
                 MOV    PLAYH,DPH
                 MOV    PLAYL,DPL
                 LCALL  PRINT                   ;provide data to flont
                 DEC    R3                      ;DECRATE DIGIT 1 DIGIT
                 MOV    BYTECOUNTRIGHT,R3       ;counter of data from key
ST1:             MOV    COUNT,#12               ;delay display
ST2:             LCALL  SUBDISPLAY              ;show display
                 DJNZ   COUNT,ST2               ;decreate of display
                 LCALL  SHRIGH                  ;shift data to right
                 JNB    MODE,SH_ROUT
                 JNB    STOPRIGHT,ST1
SH_ROUT:         CLR    STOPRIGHT
                 RET
                 ;
;*********************************************
;*             PROGRAM  SHIFT LEFT           *
;*********************************************
SH_LEFT:         CLR    STOPLEFT
                 MOV    BYTECOUNT,#1
                 MOV    DPH,PLAYH
                 MOV    DPL,PLAYL
                 MOV    A,PLAYBUF
                 ADD    A,DPL
                 MOV    DPL,A
                 MOV    A,DPH
                 ADDC   A,#0
                 MOV    DPH,A
                 MOV    PLAYH,DPH
                 MOV    PLAYL,DPL
                 LCALL  PRINT
                 MOV    COUNTER_LIMIT,R3
ST3:             MOV    COUNTL,#12
ST5:             LCALL  SUBDISPLAY
                 DJNZ   COUNTL,ST5
                 LCALL  SHLEFT
                 JNB    MODE,SH_LOUT
                 JNB    STOPLEFT,ST3
SH_LOUT:         CLR    STOPLEFT
                 RET
                 ;
;*****************************************
;*     SHIFT DATA FROM RIGH TO LEFT      *
;*****************************************
SHLEFT:     MOV    R4,#0              ;pointer to row
SHIF2:      MOV    R5,#10             ;shift 11 digit start digit 11
SHIF1:      MOV    A,R5
            MOV    B,#8               ;8 byte per digit
            MUL    AB
            ADD    A,R4
            LCALL  SLEFT
            DJNZ   R5,SHIF1
            MOV    A,R4
            INC    R4
            LCALL  SLEFT
            CLR    CTEMP
            MOV    A,R4
            CJNE   A,#8,SHIF2
            ;
            DJNZ   BYTESTEP,LEFTOUT
            MOV    BYTESTEP,#08
            MOV    R3,#8
            MOV    DPTR,#DISBUF+80
LOOPLEFT:   PUSH   DPH
            PUSH   DPL
            MOV    A,BYTECOUNT
            MOV    B,#8
            MUL    AB
            ADD    A,DPL
            MOV    DPL,A
            MOV    A,B
            ADDC   A,DPH
            MOV    DPH,A
            MOVX   A,@DPTR
            POP    DPL
            POP    DPH
            MOVX   @DPTR,A
            INC    DPTR
            DJNZ   R3,LOOPLEFT
            ;
            INC    BYTECOUNT
            MOV    A,BYTECOUNT
            CJNE   A,COUNTER_LIMIT,LEFTOUT
            MOV    BYTECOUNT,#1
            SETB   STOPLEFT
LEFTOUT:    RET
;            ;
;****************************************************
;*  PROGRAM SHIFT DATA FROM DIGIT 11 TO DIGIT 0     *
;*  DPTR = pointer at start address                 *
;*  A,B  = add dptr to point each digit             *
;*  program shift left is shift data from buffer 11 *
;*  step 1 bit by RLC A to buffer1 and increate row *
;****************************************************
SLEFT:      MOV    DPTR,#DISBUF        ;digit 0
            ADD    A,DPL
            MOV    DPL,A
            MOV    A,DPH
            ADDC   A,B
            MOV    DPH,A
            MOVX   A,@DPTR             ;get data provide shift left
            MOV    C,CTEMP             ;get carry
            RLC    A                   ;shift left
            MOV    CTEMP,C             ;save carry
            MOVX   @DPTR,A             ;put data
            RET
            ;
;********************************
;*     PROGRAM SHIFT RIGHT      *
;********************************
SHRIGH:     MOV    R4,#0               ;row
SHIFR2:     MOV    R5,#0               ;colume
SHIFR1:     MOV    A,R5
            MOV    B,#8
            MUL    AB
            ADD    A,R4
            LCALL  SRIGH               ;shift right
            INC    R5                  ;inc colume
            MOV    A,R5
            CJNE   A,#11,SHIFR1        ;start buffer - 8  shift 11 buffer
            INC    R4                  ;inc row
            CLR    CTEMP               ;end of row
            MOV    A,R4
            CJNE   A,#8,SHIFR2         ;check 8 row
            ;
NONFREE:    DJNZ   BYTESTEPRIGHT,RIHTOUT
            MOV    BYTESTEPRIGHT,#08
            MOV    R3,#8               ;1 digit = 8 byte
            MOV    DPTR,#DISBUF-8      ;display buffer start before 8 byte
LOOPRITH:   PUSH   DPH                 ;save address of before buffer dis
            PUSH   DPL                 ;save address of before buffer dis
            MOV    A,BYTECOUNTRIGHT
            MOV    B,#8
            MUL    AB
            ADD    A,DPL
            MOV    DPL,A
            MOV    A,B
            ADDC   A,DPH
            MOV    DPH,A
            MOVX   A,@DPTR             ;get data from buffer
            POP    DPL
            POP    DPH
            PUSH   ACC
            MOV    A,BYTECOUNTRIGHT
            CJNE   A,#11,RIGHT0
RIGHT0:     JNB    CY,RIGHT1
            POP    ACC
            MOV    A,#00H
            LJMP   RIGHT2
RIGHT1:     POP    ACC
RIGHT2:     MOVX   @DPTR,A             ;push data to buffer before display
            INC    DPTR                ;next colume but in digit
            DJNZ   R3,LOOPRITH
            ;
            DEC    BYTECOUNTRIGHT
            MOV    A,BYTECOUNTRIGHT
            CJNE   A,#0FFH,RIHTOUT
            SETB   STOPRIGHT
RIHTOUT:    RET
            ;
;********************************************
;*  PROGRAM SHIFT DATA FROM RIGHT TO LEFT   *
;*  DPTR =  ditgit start display            *
;*  AB   =  add dptr to point each digit    *
;********************************************
SRIGH:      CLR    CY
            MOV    DPTR,#DISBUF - 8    ;point start address
            ADD    A,DPL
            MOV    DPL,A
            MOV    A,DPH
            ADDC   A,B
            MOV    DPH,A
            MOVX   A,@DPTR             ;get data
            MOV    C,CTEMP             ;get carry
            RRC    A                   ;shift right
            MOV    CTEMP,C             ;save carry
            MOVX   @DPTR,A             ;save data to address
            RET
            ;
;*********************************
;*            SHIFT UP           *
;*********************************
SH_UP:      MOV    DPH,PLAYH
            MOV    DPL,PLAYL
            MOV    A,PLAYBUF
            ADD    A,DPL
            MOV    DPL,A
            MOV    A,DPH
            ADDC   A,#0
            MOV    DPH,A
            MOV    PLAYH,DPH
            MOV    PLAYL,DPL
            LCALL  READY
SHUP:       MOV    R3,#8
SHUP2:      MOV    R5,#0              ;digit 0
SHUP1:      MOV    A,R5
            MOV    B,#8               ;8 row  to pointer
            MUL    AB
            LCALL  SUP
            JNB    MODE,SHUPOUT
            MOV    A,R5
            INC    R5
            CJNE   A,#10,SHUP1
            MOV    R6,#12
            PUSH   03H
SHUP3:      LCALL  SUBDISPLAY
            DJNZ   R6,SHUP3
            POP    03H
            DJNZ   R3,SHUP2
SHUPOUT:    RET
;********************************
;*          SUBSHIFT UP         *
;********************************
SUP:        MOV    R4,#7              ;7 row,LEFT AND UP
            MOV    DPTR,#DISBUF       ;start disbuffer
            ADD    A,DPL              ;point digit
            MOV    DPL,A
            MOV    A,DPH
            ADDC   A,#0
            MOV    DPH,A
            ;
SUP1:       PUSH   DPH                ;save destenation  high byte
            PUSH   DPL                ;save destenation  low  byte
            INC    DPTR               ;point to source
            MOVX   A,@DPTR            ;get data from source
            POP    DPL                ;get destination high
            POP    DPH                ;get destenation low
            PUSH   ACC
            MOV    A,R4
            CJNE   A,#1,SUP2
            POP    ACC
            MOVX   @DPTR,A
            MOV    A,#00
            INC    DPTR
            SJMP   SUP3
SUP2:       POP    ACC
SUP3:       MOVX   @DPTR,A            ;send data to destenation
            INC    DPTR               ;increate point source
            DJNZ   R4,SUP1
            RET
            ;
;********************************
;*           SHDOWN             *
;********************************
SH_DOWN:    MOV    DPH,PLAYH
            MOV    DPL,PLAYL
            MOV    A,PLAYBUF
            ADD    A,DPL
            MOV    DPL,A
            MOV    A,DPH
            ADDC   A,#0
            MOV    DPH,A
            MOV    PLAYH,DPH
            MOV    PLAYL,DPL
            LCALL  READY
            ;
            MOV    R3,#8
SHDOWN:     MOV    R5,#0
SHDOWN1:    MOV    A,R5
            MOV    B,#8
            MUL    AB
            LCALL  SDOWN
            JNB    MODE,SHDOUT
            MOV    A,R5
            INC    R5
            CJNE   A,#10,SHDOWN1
            MOV    R6,#12
            PUSH   03H
DOWN1:      LCALL  SUBDISPLAY
            DJNZ   R6,DOWN1
            POP    03H
            DJNZ   R3,SHDOWN
SHDOUT:     RET
            ;
SDOWN:      MOV    R4,#7
            MOV    DPTR,#DISBUF+7
            ADD    A,DPL
            MOV    DPL,A
            MOV    A,DPH
            ADDC   A,#0
            MOV    DPH,A
SDOWN1:     PUSH   DPH
            PUSH   DPL
            ;
            CLR    CY
            MOV    A,DPL
            SUBB   A,#01
            MOV    DPL,A
            MOV    A,DPH
            ADDC   A,#0
            MOV    DPH,A
            MOVX   A,@DPTR
            POP    DPL
            POP    DPH
            MOVX   @DPTR,A
            ;
            CLR    CY
            MOV    A,DPL
            SUBB   A,#01
            MOV    DPL,A
            MOV    A,DPH
            SUBB   A,#0
            MOV    DPH,A
            DJNZ   R4,SDOWN1
            MOV    A,#00
            MOVX   @DPTR,A
            RET
            ;
;***************************************
;*           READY PROGRAM             *
;***************************************
READY:       LCALL  PRINT
             ;
             MOV    R6,#80
             MOV    DPTR,#DISBUF
BLANDIS1:    PUSH   DPH
             PUSH   DPL
             MOV    A,DPL
             ADD    A,#80
             MOV    DPL,A
             MOV    A,DPH
             ADDC   A,#00
             MOV    DPH,A
             MOVX   A,@DPTR
             POP    DPL
             POP    DPH
             MOVX   @DPTR,A
             INC    DPTR
             DJNZ   R6,BLANDIS1
             RET
             ;
;***************************************
;*           READY PROGRAM             *
;***************************************
READYX:      LCALL  PRINTX
             ;
             MOV    R6,#80
             MOV    DPTR,#DISBUF
BLANDISX1:   PUSH   DPH
             PUSH   DPL
             MOV    A,DPL
             ADD    A,#80
             MOV    DPL,A
             MOV    A,DPH
             ADDC   A,#00
             MOV    DPH,A
             MOVX   A,@DPTR
             POP    DPL
             POP    DPH
             MOVX   @DPTR,A
             INC    DPTR
             DJNZ   R6,BLANDISX1
             RET
             ;
;FLASHDIS:    MOV    DPTR,#DOWNBUF
;             LCALL  PRINT
             ;
;FLASH:       MOV    R7,#10
;FLASH2:      MOV    R6,#80
;             MOV    DPTR,#DISBUF
;FLASH1:      MOV    A,#00H
;             MOVX   @DPTR,A
;             INC    DPTR
;             DJNZ   R6,FLASH1
;             MOV    R6,#12
;SUBCLR:      LCALL  SUBDISPLAY
;             DJNZ   R6,SUBCLR
;             ;
;             MOV    R6,#80
;             MOV    DPTR,#DISBUF
;SUBDIS1:     PUSH   DPH
;             PUSH   DPL
;             MOV    A,DPL
;             ADD    A,#80
;              MOV    DPL,A
 ;            MOV    A,DPH
 ;            ADDC   A,#00
 ;             MOV    DPH,A
;             MOVX   A,@DPTR
;             POP    DPL
;             POP    DPH
;             MOVX   @DPTR,A
;             INC    DPTR
;             DJNZ   R6,SUBDIS1
             ;
;             MOV    R6,#12
; SUBDIS2:     LCALL  SUBDISPLAY
 ;            DJNZ   R6,SUBDIS2
 ;            DJNZ   R7,FLASH2
 ;             RET
             ;
;****************************************
;*              LOOPWORK                *
;****************************************
LOOPWORK:     MOV     DPTR,#SEQBUF
LOOPWORK2:    JNB     MODE,WORKOUT
              MOVX    A,@DPTR
              PUSH    DPH
              PUSH    DPL
              CJNE    A,#0DH,LOOPWORK1     ;end of loop
              POP     DPL
              POP     DPH
              MOV     DPTR,#ACBUF
              MOV     PLAYH,DPH
              MOV     PLAYL,DPL
              MOV     PLAYBUF,#0
              LJMP    LOOPWORK
LOOPWORK1:    LCALL   SEQUEN               ;call program
              POP     DPL
              POP     DPH
              INC     DPTR
              LJMP    LOOPWORK2
WORKOUT:      RET
              ;
;****************************************
;*            SELECT  MENU              *
;****************************************
SEQUEN:      CJNE   A,#01,SEQ1
             SETB   INVERTBITS
             LCALL  SH_UP
             LJMP   SEQOUT
             ;
SEQ1:        CJNE   A,#02,SEQ2
             SETB   INVERTBITS
             LCALL  SH_DOWN
             LJMP   SEQOUT
             ;
SEQ2:        CJNE   A,#03,SEQ3
             SETB   INVERTBITS
             LCALL  SH_LEFT
             LJMP   SEQOUT
             ;
SEQ3:        CJNE   A,#04,SEQ5
             SETB   INVERTBITS
             LCALL  SH_RIGHT
             LJMP   SEQOUT
             ;
;SEQ4:        CJNE   A,#05H,SEQ5
;             SETB   INVERTBITS
;             LCALL  FLASHDIS
;             LJMP   SEQOUT
             ;
SEQ5:        CJNE   A,#06H,SEQ6
             CLR    INVERTBITS
             LCALL  SH_UP
             LJMP   SEQOUT
             ;
SEQ6:        CJNE   A,#07H,SEQ7
             CLR    INVERTBITS
             LCALL  SH_DOWN
             LJMP   SEQOUT
             ;
SEQ7:        CJNE   A,#08H,SEQ8
             CLR    INVERTBITS
             LCALL  SH_LEFT
             LJMP   SEQOUT
             ;
SEQ8:        CJNE   A,#09H,SEQOUT
             CLR    INVERTBITS
             LCALL  SH_RIGHT
             LJMP   SEQOUT
             ;
;SEQ9:        CJNE   A,#0AH,SEQOUT
;             CLR    INVERTBITS
;             LCALL  FLASHDIS
SEQOUT:      RET
             ;
             ;
DELAY:       PUSH   DPH
             PUSH   DPL
             MOV    DPTR,#8000H
DEL:         INC    DPTR
             MOV    A,DPH
             ORL    A,DPL
             JNZ    DEL
             POP    DPL
             POP    DPH
             RET
            ;
            ;
FONT:       DB 00H,00H,00H,00H,00H,00H,00H,00H    ; Null
            DB 10H,10H,10H,10H,00H,00H,10H,00H    ; !
            DB 28H,28H,28H,00H,00H,00H,00H,00H    ; "
            DB 28H,28H,7CH,28H,7CH,28H,28H,00H    ; #
            DB 10H,3CH,50H,38H,14H,78H,10H,00H    ; $
            DB 60H,64H,08H,10H,20H,4CH,0CH,00H    ; %
            DB 30H,48H,50H,20H,54H,48H,34H,00H    ; &
            DB 30H,10H,20H,00H,00H,00H,00H,00H    ; '
            DB 08H,10H,20H,20H,20H,10H,08H,00H    ; (
            DB 20H,10H,08H,08H,08H,10H,20H,00H    ; )
            DB 00H,10H,54H,38H,54H,10H,00H,00H    ; *
            DB 00H,10H,10H,7CH,10H,10H,00H,00H    ; +
            DB 00H,00H,00H,00H,30H,10H,20H,00H    ; ,
            DB 00H,00H,00H,7CH,00H,00H,00H,00H    ; -
            DB 00H,00H,00H,00H,00H,30H,30H,00H    ; .
            DB 00H,04H,08H,10H,20H,40H,00H,00H    ; /
            DB 38H,44H,4CH,54H,64H,44H,38H,00H    ; 0
            DB 10H,30H,10H,10H,10H,10H,38H,00H    ; 1
            DB 38H,44H,04H,08H,10H,20H,7CH,00H    ; 2
            DB 7CH,08H,10H,08H,04H,44H,38H,00H    ; 3
            DB 08H,18H,28H,48H,7CH,08H,08H,00H    ; 4
            DB 7CH,40H,78H,04H,04H,44H,38H,00H    ; 5
            DB 18H,20H,40H,78H,44H,44H,38H,00H    ; 6
            DB 7CH,04H,08H,10H,20H,20H,20H,00H    ; 7
            DB 38H,44H,44H,38H,44H,44H,38H,00H    ; 8
            DB 38H,44H,44H,3CH,04H,08H,30H,00H    ; 9
            DB 00H,30H,30H,00H,30H,30H,00H,00H    ; :
            DB 00H,30H,30H,00H,30H,10H,20H,00H    ; ;
            DB 08H,10H,20H,40H,20H,10H,08H,00H    ; <
            DB 00H,00H,7CH,00H,7CH,00H,00H,00H    ; =
            DB 20H,10H,08H,04H,08H,10H,20H,00H    ; >
            DB 38H,44H,04H,08H,10H,00H,10H,00H    ; ?
            DB 38H,44H,04H,34H,54H,54H,38H,00H    ; @
UPPER:      DB 38H,44H,44H,7CH,44H,44H,44H,00H    ; A
            DB 78H,44H,44H,78H,44H,44H,78H,00H    ; B
            DB 38H,44H,40H,40H,40H,44H,38H,00H    ; C
            DB 70H,48H,44H,44H,44H,48H,70H,00H    ; D
            DB 7CH,40H,40H,78H,40H,40H,7CH,00H    ; E
            DB 7CH,40H,40H,78H,40H,40H,40H,00H    ; F
            DB 38H,44H,40H,5CH,44H,44H,3CH,00H    ; G
            DB 44H,44H,44H,7CH,44H,44H,44H,00H    ; H
            DB 38H,10H,10H,10H,10H,10H,38H,00H    ; I
            DB 1CH,08H,08H,08H,08H,48H,30H,00H    ; J
            DB 44H,48H,50H,60H,50H,48H,44H,00H    ; K
            DB 40H,40H,40H,40H,40H,40H,7CH,00H    ; L
            DB 44H,6CH,54H,54H,44H,44H,44H,00H    ; M
            DB 44H,44H,64H,54H,4CH,44H,44H,00H    ; N
            DB 38H,44H,44H,44H,44H,44H,38H,00H    ; O
            DB 78H,44H,44H,78H,40H,40H,40H,00H    ; P
            DB 38H,44H,44H,44H,54H,48H,34H,00H    ; Q
            DB 78H,44H,44H,78H,50H,48H,44H,00H    ; R
            DB 3CH,40H,40H,38H,04H,04H,78H,00H    ; S
            DB 7CH,10H,10H,10H,10H,10H,10H,00H    ; T
            DB 44H,44H,44H,44H,44H,44H,38H,00H    ; U
            DB 44H,44H,44H,44H,44H,28H,10H,00H    ; V
            DB 44H,44H,44H,54H,54H,54H,28H,00H    ; W
            DB 44H,44H,28H,10H,28H,44H,44H,00H    ; X
            DB 44H,44H,44H,28H,10H,10H,10H,00H    ; Y
            DB 7CH,04H,08H,10H,20H,40H,7CH,00H    ; Z
            DB 38H,20H,20H,20H,20H,20H,38H,00H    ; [
            DB 40H,20H,10H,08H,04H,02H,01H,00H    ; \
            DB 38H,08H,08H,08H,08H,08H,38H,00H    ; ]
            DB 10H,28H,44H,00H,00H,00H,00H,00H    ; ^
            DB 00H,00H,00H,00H,00H,00H,7CH,00H    ; _
            DB 20H,10H,08H,00H,00H,00H,00H,00H    ;`
LOWER:      DB 00H,00H,38H,08H,38H,48H,34H,00H    ; a
            DB 40H,40H,58H,64H,44H,44H,78H,00H    ; b
            DB 00H,00H,3CH,40H,40H,40H,3CH,00H    ; c
            DB 04H,04H,34H,4CH,44H,44H,3CH,00H    ; d
            DB 00H,00H,38H,44H,78H,40H,38H,00H    ; e
            DB 18H,24H,20H,70H,20H,20H,20H,00H    ; f
            DB 00H,00H,38H,44H,44H,3CH,04H,38H    ; g
            DB 40H,40H,58H,64H,44H,44H,44H,00H    ; h
            DB 10H,00H,30H,10H,10H,10H,38H,00H    ; i
            DB 00H,08H,00H,08H,08H,28H,10H,00H    ; j
            DB 40H,40H,48H,50H,60H,50H,48H,00H    ; k
            DB 30H,10H,10H,10H,10H,10H,18H,00H    ; l
            DB 00H,00H,68H,54H,54H,54H,44H,00H    ; m
            DB 00H,00H,58H,24H,24H,24H,24H,00H    ; n
            DB 00H,00H,38H,44H,44H,44H,38H,00H    ; o
            DB 00H,00H,70H,48H,48H,70H,40H,40H    ; p
            DB 00H,00H,38H,48H,48H,38H,08H,08H    ; q
            DB 00H,00H,58H,64H,40H,40H,40H,00H    ; r
            DB 00H,00H,38H,40H,38H,04H,78H,00H    ; s
            DB 20H,20H,78H,20H,20H,24H,18H,00H    ; t
            DB 00H,00H,48H,48H,48H,48H,34H,00H    ; u
            DB 00H,00H,44H,44H,28H,28H,10H,00H    ; v
            DB 00H,00H,44H,44H,54H,54H,28H,00H    ; w
            DB 00H,00H,44H,28H,10H,28H,44H,00H    ; x
            DB 00H,00H,48H,48H,48H,38H,08H,70H    ; y
            DB 00H,00H,7CH,08H,10H,20H,7CH,00H    ; z
            DB 08H,10H,10H,20H,10H,10H,08H,00H    ; {
            DB 18H,18H,18H,18H,18H,18H,18H,00H    ; |
            DB 20H,10H,10H,08H,10H,10H,20H,00H    ; }
            DB 00H,00H,00H,32H,4CH,00H,00H,00H    ; ~
            ;
            ORG    100
DISBUF      EQU    100
PDISBUF     EQU    180
            ;
            ORG    2500
ACBUF:      DS     5300
            ;
            ORG    7900
SEQBUF:     DS     256
            ;
            END




